<!-- 
     Extract XMLdata from AS/400 data stream:
     00 00 00 01  32bitlen  00 00 00 00  XMLdata
-->
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:func="http://exslt.org/functions"
  xmlns:dp="http://www.datapower.com/extensions"
  extension-element-prefixes="dp"
>
  <dp:input-mapping href="store:///pkcs7-convert-input.ffd" type="ffd"/>
  <!-- 
    This FFD converts the input into an XML tree like this:

    <object>
      <message>***binary data***</message>
    </object>
  -->

  <xsl:output omit-xml-declaration="yes" />

  <xsl:template match="/">
    <!-- complete input as base64 encoded string -->
    <xsl:variable name="all" select="dp:binary-encode(/object/message)" />

    <!-- 1 -->
    <xsl:if test="dp:radix-convert(dp:substring-base64($all,1,4),64,10)!=1">
      <xsl:message terminate="yes">1st integer not 1</xsl:message>
    </xsl:if>

    <!-- 2 -->
    <!-- serialized XML input, inclusive UTF-8 validation -->
    <xsl:variable name="serialized" 
      select="dp:decode(dp:substring-base64($all,13),'base-64')" />

    <!-- 3 -->
    <xsl:if test="dp:radix-convert(dp:substring-base64($all,5,4),64,10)
                  != string-length($serialized)">
      <xsl:message terminate="yes">XML length does not match</xsl:message>
    </xsl:if>

    <!-- 4 -->
    <xsl:if test="dp:radix-convert(dp:substring-base64($all,9,4),64,10)!=0">
      <xsl:message terminate="yes">3rd integer not 0</xsl:message>
    </xsl:if>

    <xsl:variable name="xml">
      <xsl:copy-of select="dp:parse($serialized)" />
    </xsl:variable>

    <!-- 5 -->
    <xsl:if test="not($xml/*)">
      <xsl:message terminate="yes">Non-XML payload</xsl:message>
    </xsl:if>

    <xsl:copy-of select="$xml"/>
  </xsl:template>
    
</xsl:stylesheet>
