<!-- 
     Convert input XML to AS/400 data stream:
     00 00 00 01  32bitlen  00 00 00 00  XMLdata
-->
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:func="http://exslt.org/functions"
  xmlns:dp="http://www.datapower.com/extensions"
  extension-element-prefixes="dp"
>
  <dp:output-mapping href="store:///pkcs7-convert-input.ffd" type="ffd"/>
  <!-- 
    This FFD converts the XML tree like this to binary output:

    <object>
      <message>***binary data***</message>
    </object>
  -->

  <xsl:output omit-xml-declaration="yes" />

  <!-- 
       convert unsigned number to base64 encoded 32bit integer.

       258 --> AAABAg==  (= 00 00 01 02)
  -->
  <func:function name="func:uint2b64">
    <xsl:param name="u"/>
    
    <func:result
      select="dp:substring-base64(
                dp:radix-convert($u+4294967296,10,64),
                2
              )"/>
  </func:function>


  <xsl:template match="/">
    <!-- Serialize XML input -->
    <xsl:variable name="serialized">
      <dp:serialize select="." omit-xml-decl="yes"/>
    </xsl:variable>

    <!-- determine AS/400 encoding -->
    <xsl:variable name="b64"
      select="dp:concat-base64(
                dp:concat-base64(
                  func:uint2b64(1),
                  func:uint2b64(string-length($serialized))
                ),
                dp:concat-base64(
                  func:uint2b64(0),
                  dp:encode($serialized,'base-64')
                )
              )"
    />

    <!-- prepare binaryNode return for dp:output-mapping -->
    <object>
      <message>
        <xsl:copy-of 
          select="dp:binary-decode($b64)" />
      </message>
    </object>
  </xsl:template>
    
</xsl:stylesheet>
