<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dp="http://www.datapower.com/extensions"
  xmlns:func="http://exslt.org/functions"
  extension-element-prefixes="dp" 
>
  <dp:input-mapping href="store:///pkcs7-convert-input.ffd" type="ffd"/>

<!-- 
  expandKey2 selects "YyyyyyyZ" from
    xxxY|yyyy|yyZz for $div=2
    xxYy|yyyy|yZzz for $div=4
    xYyy|yyyy|Zzzz for $div=8
-->
  <func:function name="func:expandKey2">
    <xsl:param name="hex"/>
    <xsl:param name="div"/>

    <!-- make sure we have 3 bytes before "shift right" -->
    <xsl:variable name="num" 
      select="0x10000 + dp:radix-convert($hex,16,10)"/>

    <xsl:variable name="shr" 
      select="floor($num div $div)"/>

    <!-- get last(2nd) byte only -->
    <func:result select="substring(dp:radix-convert($shr,10,16),3)"/>
  </func:function>

<!-- 
  expandKey creates 64bit key from 56bit key presented as hex string 

  (last bits should be parity, but DES does not care, so take it easy)
        01  02   03  04   05  06   07  08   09  10   11  12   13  14
   in: AaaaaaaB|bbbbbbCc|cccccDdd|ddddEeee|eeeFffff|ffGggggg|gHhhhhhh
  out: AaaaaaaB|BbbbbbbC|CccccccD|DddddddE|EeeeeeeF|FffffffG|GggggggH|Hhhhhhh0
-->
  <func:function name="func:expandKey">
    <xsl:param name="key"/>

    <xsl:variable name="last" select="concat(substring($key,13,2),'0')"/>

    <func:result> 
      <xsl:value-of select="substring($key,1,2)"/>
      <xsl:value-of select="func:expandKey2(substring($key,2,3),2)"/>
      <xsl:value-of select="func:expandKey2(substring($key,4,3),4)"/>
      <xsl:value-of select="func:expandKey2(substring($key,6,3),8)"/>
      <xsl:value-of select="substring($key,8,2)"/>
      <xsl:value-of select="func:expandKey2(substring($key,9,3),2)"/>
      <xsl:value-of select="func:expandKey2(substring($key,11,3),4)"/>
      <xsl:value-of select="func:expandKey2($last,8)"/>
    </func:result>
  </func:function>

<!--
  read in 56bit DES key and convert to 64bit DES key with "parity" bits
-->
  <xsl:template match="/">

    <xsl:variable name="input64" 
      select="dp:binary-encode(/object/message/node())"/>

    <xsl:variable name="hex"
      select="substring(dp:radix-convert(concat('8AAA',$input64),64,16),7)"/>

    <xsl:value-of select="$hex"/>
    <xsl:text>&#10;</xsl:text>
    <xsl:value-of select="func:expandKey($hex)"/>
  </xsl:template>

</xsl:stylesheet>
