<!DOCTYPE xsl:stylesheet [ <!ENTITY LF "<xsl:text>&#10;</xsl:text>"> ]>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:func="http://exslt.org/functions"
  xmlns:dp="http://www.datapower.com/extensions"
>
  <dp:input-mapping  href="store:///pkcs7-convert-input.ffd" type="ffd"/>

  <xsl:include href="map.xsl"/>
 
  <func:function name="func:pbm-ascii">
    <func:result 
      select="dp:decode(dp:binary-encode(/object/message/node()),'base-64')"/>
  </func:function>


  <xsl:template match="token">
    <xsl:param name="xpos" select="1"/>

    <xsl:if test="string-length() > $xpos">
      <xsl:call-template name="char">
        <xsl:with-param name="chr" 
          select="concat(substring(.,$xpos,2),
                         substring(following-sibling::*[1],$xpos,2),
                         substring(following-sibling::*[2],$xpos,2),
                         substring(following-sibling::*[3],$xpos,2))"/>
      </xsl:call-template>

      <xsl:apply-templates select=".">
        <xsl:with-param name="xpos" select="$xpos + 2"/>
      </xsl:apply-templates>
    </xsl:if>

    <xsl:if test="not(string-length() > $xpos)">&LF;</xsl:if>
  </xsl:template>


  <xsl:template name="formatCheck">
    <xsl:param name="format"/>
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="lines" select="/.."/>

    <xsl:if test="$format != 'P1'">
      <xsl:message dp:priority="error" 
        terminate="yes">only ASCII pbm format (P1) supported</xsl:message>
    </xsl:if>

    <xsl:if test="$width > 70">
      <xsl:message dp:priority="error" 
        terminate="yes">no support for width > 70</xsl:message>
    </xsl:if>
   
    <xsl:if test="$width mod 2 = 1">
      <xsl:message dp:priority="error" 
        terminate="yes">no support for odd width</xsl:message>
    </xsl:if>
   
    <xsl:if test="not($height mod 4 = 0)">
      <xsl:message dp:priority="error" 
        terminate="yes">no support for non-quad height</xsl:message>
    </xsl:if>
   
    <xsl:for-each select="$lines">
      <xsl:if test="string-length() != $width">
        <xsl:message dp:priority="error" 
          terminate="yes">all lines must match width</xsl:message>
      </xsl:if>
    </xsl:for-each>

    <xsl:if test="count($lines) != $height">
      <xsl:message dp:priority="error" 
        terminate="yes">line count must match height</xsl:message>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
